@@ -1,3 +1,4 @@
+// -*- c-basic-offset: 4; indent-tabs-mode: nil; tab-width: 4; -*-'
 //Author: Dodji Seketeli
 /*
  *This file is part of the Nemiver project
@@ -179,7 +180,6 @@ class GDBMIList : public Object {
     GDBMIList (const GDBMIList &);
     GDBMIList& operator= (const GDBMIList &);
 
-    //boost::variant<list<GDBMIResultSafePtr>, list<GDBMIValueSafePtr> > m_content;
     list<boost::variant<GDBMIResultSafePtr, GDBMIValueSafePtr> >  m_content;
     bool m_empty;
 
@@ -325,6 +325,13 @@ class GDBMIParser {
     struct Priv;
     SafePtr<Priv> m_priv;
 
+    bool analyse_pure_asm_instrs (GDBMIListSafePtr,
+                                    list<common::AsmInstr>&,
+                                    string::size_type a_cur);
+
+    bool analyse_mixed_asm_instrs (GDBMIListSafePtr,
+                                      list<common::MixedAsmInstr>&,
+                                      string::size_type a_cur);
 public:
 
     /// Parsing mode.
@@ -515,11 +522,11 @@ public:
     /// \return true in case of successful parsing, false otherwise.
     bool parse_breakpoint (Glib::ustring::size_type a_from,
                            Glib::ustring::size_type &a_to,
-                           IDebugger::BreakPoint &a_bkpt);
+                           IDebugger::Breakpoint &a_bkpt);
 
     bool parse_breakpoint_table (UString::size_type a_from,
                                  UString::size_type &a_to,
-                                 map<int, IDebugger::BreakPoint> &a_breakpoints);
+                                 map<int, IDebugger::Breakpoint> &a_breakpoints);
 
     /// parses the result of the gdbmi command
     /// "-thread-list-ids".
@@ -560,7 +567,8 @@ public:
     /// See the GDB/MI documentation for more.
     bool parse_stack_arguments (UString::size_type a_from,
                                 UString::size_type &a_to,
-                                map<int, list<IDebugger::VariableSafePtr> > &a_params);
+                                map<int,
+                                    list<IDebugger::VariableSafePtr> > &a_parms);
 
     /// parse a list of local variables as returned by
     /// the GDBMI command -stack-list-locals 2
@@ -593,17 +601,24 @@ public:
 
     bool parse_changed_registers (UString::size_type a_from,
                                   UString::size_type &a_to,
-                                  std::list<IDebugger::register_id_t> &a_registers);
+                                  std::list<IDebugger::register_id_t> &a_regs);
 
     bool parse_register_values (UString::size_type a_from,
                                 UString::size_type &a_to,
-                                std::map<IDebugger::register_id_t, UString> &a_values);
+                                std::map<IDebugger::register_id_t,
+                                         UString> &a_values);
 
     bool parse_memory_values (UString::size_type a_from,
                               UString::size_type &a_to,
                               size_t& a_start_addr,
                               std::vector<uint8_t> &a_values);
 
+    /// parse an asm instruction description as returned
+    /// by GDB/MI
+    bool parse_asm_instruction_list (UString::size_type a_from,
+                                     UString::size_type &a_to,
+                                     std::list<common::Asm> &a_asm);
+
     bool parse_variable (UString::size_type a_from,
                          UString::size_type &a_to,
                          IDebugger::VariableSafePtr &a_var);
@@ -633,6 +648,9 @@ public:
                               UString::size_type &a_to,
                               Output &a_output);
 
+    bool skip_output_record (UString::size_type a_from,
+			     UString::size_type &a_to);
+
     //*********************
     //</Parsing entry points.>
     //*********************
