@@ -22,12 +22,16 @@
  *
  *See COPYRIGHT file copyright information.
  */
+
+#include <iostream>
+
 #include "common/nmv-safe-ptr-utils.h"
 #include "common/nmv-connection.h"
 #include "common/nmv-connection-manager.h"
 #include "common/nmv-tools.h"
 #include "common/nmv-transaction.h"
 #include "common/nmv-sql-statement.h"
+#include "common/nmv-conf-manager.h"
 #include "nmv-ui-utils.h"
 #include "nmv-sess-mgr.h"
 
@@ -39,7 +43,8 @@ using nemiver::common::ConnectionManager;
 using nemiver::common::Transaction;
 using nemiver::common::SQLStatement;
 
-static const char *REQUIRED_DB_SCHEMA_VERSION = "1.3";
+static const char *REQUIRED_DB_SCHEMA_VERSION = "1.5";
+static const char *DB_FILE_NAME = "nemivercommon.db";
 
 NEMIVER_BEGIN_NAMESPACE (nemiver)
 
@@ -89,7 +94,7 @@ struct SessMgr::Priv {
     ConnectionSafePtr connection ()
     {
         if (!conn) {
-            conn= ConnectionManager::create_db_connection ();
+            conn = ConnectionManager::create_db_connection ();
         }
         THROW_IF_FAIL (conn);
         return conn;
@@ -114,12 +119,34 @@ struct SessMgr::Priv {
         return Glib::locale_to_utf8 (path);
     }
 
+    const string& get_db_file_path () const
+    {
+        static string db_file_path;
+        if (db_file_path.empty ()) {
+            vector<string> path_elems;
+            path_elems.push_back (ConfManager::get_user_config_dir_path ());
+            path_elems.push_back (DB_FILE_NAME);
+            db_file_path = Glib::build_filename (path_elems);
+        }
+        LOG_DD ("db_file_path: " << db_file_path);
+        return db_file_path;
+    }
+
+    bool db_file_path_exists () const
+    {
+        if (Glib::file_test (get_db_file_path (), Glib::FILE_TEST_EXISTS)) {
+            LOG_DD ("could not find file: " << get_db_file_path ());
+            return true;
+        }
+        return false;
+    }
+
     bool create_db ()
     {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
         UString path_to_script = path_to_create_tables_script ();
-        Transaction transaction (*conn);
+        Transaction transaction (*connection ());
         return tools::execute_sql_command_file
                                         (path_to_script,
                                          transaction,
@@ -131,7 +158,7 @@ struct SessMgr::Priv {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
         UString path_to_script = path_to_drop_tables_script ();
-        Transaction transaction (*conn);
+        Transaction transaction (*connection ());
         return tools::execute_sql_command_file
                                         (path_to_script,
                                          transaction,
@@ -163,9 +190,14 @@ struct SessMgr::Priv {
         LOG_FUNCTION_SCOPE_NORMAL_DD;
 
         NEMIVER_TRY
-        //if the db version is not what we expect, create
-        //a new db with the schema we expect.
-        if (!check_db_version ()) {
+
+        // If there is not db, create a new one with 
+        // the schema we expect.
+        if (!db_file_path_exists ()) {
+            THROW_IF_FAIL (create_db ());
+        } else if (!check_db_version ()) {
+            // If the db version is not what we expect, create
+            // a new db with the schema we expect.
             drop_db ();
             THROW_IF_FAIL (create_db ());
         }
@@ -230,15 +262,15 @@ SessMgr::store_session (Session &a_session,
 {
     THROW_IF_FAIL (m_priv);
 
-    //The next line starts a transaction.
-    //If we get off from this function without reaching
-    //the trans.end() call, every db request we made gets rolled back.
+    // The next line starts a transaction.
+    // If we get off from this function without reaching
+    // the trans.end() call, every db request we made gets rolled back.
     TransactionAutoHelper trans (a_trans);
 
     UString query;
     if (!a_session.session_id ()) {
-        //insert the session id in the sessions table, and get the session id
-        //we just inerted
+        // insert the session id in the sessions table, and get the session id
+        // we just inerted
         query = "insert into sessions values(NULL)";
         THROW_IF_FAIL2
             (trans.get ().get_connection ().execute_statement (query),
@@ -257,7 +289,7 @@ SessMgr::store_session (Session &a_session,
         a_session.session_id (session_id);
     }
 
-    //store the properties
+    // store the properties
     query = "delete from attributes where sessionid = "
             + UString::from_int (a_session.session_id ());
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
@@ -268,15 +300,15 @@ SessMgr::store_session (Session &a_session,
          ++prop_iter) {
         query = "insert into attributes values(NULL, "
                 + UString::from_int (a_session.session_id ()) + ", '"
-                + prop_iter->first + "', '"
-                + prop_iter->second
+                + SQLStatement::escape_string (prop_iter->first) + "', '"
+                + SQLStatement::escape_string (prop_iter->second)
                 + "')";
         LOG_DD ("query: " << query);
         THROW_IF_FAIL
                 (trans.get ().get_connection ().execute_statement (query));
     }
 
-    //store the environment variables
+    // store the environment variables
     query = "delete from env_variables where sessionid = "
             + UString::from_int (a_session.session_id ());
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
@@ -287,39 +319,66 @@ SessMgr::store_session (Session &a_session,
          ++var_iter) {
         query = "insert into env_variables values(NULL, "
                 + UString::from_int (a_session.session_id ()) + ", '"
-                + var_iter->first + "', '"
-                + var_iter->second
+                + SQLStatement::escape_string (var_iter->first) + "', '"
+                + SQLStatement::escape_string (var_iter->second)
                 + "')";
         LOG_DD ("query: " << query);
         THROW_IF_FAIL
                 (trans.get ().get_connection ().execute_statement (query));
     }
 
-    //store the breakpoints
+    // store the breakpoints
     query = "delete from breakpoints where sessionid = "
             + UString::from_int (a_session.session_id ());
     THROW_IF_FAIL
             (trans.get ().get_connection ().execute_statement (query));
 
-    list<SessMgr::BreakPoint>::const_iterator break_iter;
+    list<SessMgr::Breakpoint>::const_iterator break_iter;
     for (break_iter = a_session.breakpoints ().begin ();
          break_iter != a_session.breakpoints ().end ();
          ++break_iter) {
         UString condition = break_iter->condition ();
         condition.chomp ();
         query = "insert into breakpoints values(NULL, "
-                + UString::from_int (a_session.session_id ()) + ", '"
-                + break_iter->file_name () + "', '"
-                + break_iter->file_full_name () + "', "
-                + UString::from_int (break_iter->line_number ()) + ", "
-                + UString::from_int (break_iter->enabled ()) + ", "
-                + "'" + condition + "')";
+	  + UString::from_int (a_session.session_id ()) + ", '"
+	  + SQLStatement::escape_string (break_iter->file_name ()) + "', '"
+	  + SQLStatement::escape_string
+	  (break_iter->file_full_name ()) + "', "
+	  + UString::from_int (break_iter->line_number ()) + ", "
+	  + UString::from_int (break_iter->enabled ()) + ", "
+	  + "'" + SQLStatement::escape_string (condition) + "'" + ", "
+	  + UString::from_int (break_iter->ignore_count ()) + ", "
+	  + UString::from_int (break_iter->is_countpoint ())
+	  + ")";
         LOG_DD ("query: " << query);
         THROW_IF_FAIL
                 (trans.get ().get_connection ().execute_statement (query));
     }
 
-    //store the opened files
+    // store the watchpoints
+    query = "delete from watchpoints where sessionid = "
+            + UString::from_int (a_session.session_id ());
+    THROW_IF_FAIL
+            (trans.get ().get_connection ().execute_statement (query));
+
+    list<SessMgr::WatchPoint>::const_iterator watch_iter;
+    for (watch_iter = a_session.watchpoints ().begin ();
+         watch_iter != a_session.watchpoints ().end ();
+         ++watch_iter) {
+        UString expression = watch_iter->expression ();
+        expression.chomp ();
+        query = "insert into watchpoints values(NULL, "
+                + UString::from_int (a_session.session_id ()) + ", '"
+                + SQLStatement::escape_string (expression) + "', "
+                + UString::from_int (watch_iter->is_read ()) + ", "
+                + UString::from_int (watch_iter->is_write ())
+                + ")";
+        LOG_DD ("query: " << query);
+        THROW_IF_FAIL
+                (trans.get ().get_connection ().execute_statement (query));
+    }
+
+    // store the opened files
     query = "delete from openedfiles where sessionid = "
             + UString::from_int (a_session.session_id ());
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
@@ -330,14 +389,14 @@ SessMgr::store_session (Session &a_session,
          ++ofile_iter) {
         query = "insert into openedfiles values(NULL, "
                 + UString::from_int (a_session.session_id ()) + ", '"
-                + *ofile_iter
+                + SQLStatement::escape_string (*ofile_iter)
                 + "')";
         LOG_DD ("query: " << query);
         THROW_IF_FAIL
                 (trans.get ().get_connection ().execute_statement (query));
     }
 
-    //store the search paths
+    // store the search paths
     query = "delete from searchpaths where sessionid = "
             + UString::from_int (a_session.session_id ());
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
@@ -348,7 +407,7 @@ SessMgr::store_session (Session &a_session,
          ++path_iter) {
         query = "insert into searchpaths values(NULL, "
                 + UString::from_int (a_session.session_id ()) + ", '"
-                + *path_iter
+                + SQLStatement::escape_string (*path_iter)
                 + "')";
         LOG_DD ("query: " << query);
         THROW_IF_FAIL
@@ -381,15 +440,16 @@ SessMgr::load_session (Session &a_session,
     Session session;
     session.session_id (a_session.session_id ());
 
-    //The next line starts a transaction.
-    //If we get off from this function without reaching
-    //the trans.end() call, every db request we made gets rolled back.
+    // The next line starts a transaction.
+    // If we get off from this function without reaching
+    // the trans.end() call, every db request we made gets rolled back.
     TransactionAutoHelper trans (a_trans);
 
-    //load the attributes
+    // load the attributes
     UString query="select attributes.name, attributes.value "
                   "from attributes where attributes.sessionid = "
                   + UString::from_int (a_session.session_id ());
+    LOG_DD ("query: " << query);
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
     while (trans.get ().get_connection ().read_next_row ()) {
         UString name, value;
@@ -400,10 +460,11 @@ SessMgr::load_session (Session &a_session,
         session.properties ()[name] = value;
     }
 
-    //load the environment variables
+    // load the environment variables
     query = "select env_variables.name, env_variables.value "
             "from env_variables where env_variables.sessionid = "
                   + UString::from_int (a_session.session_id ());
+    LOG_DD ("query: " << query);
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
     while (trans.get ().get_connection ().read_next_row ()) {
         UString name, value;
@@ -414,15 +475,19 @@ SessMgr::load_session (Session &a_session,
         session.env_variables ()[name] = value;
     }
 
-    //load the breakpoints
+    // load the breakpoints
     query = "select breakpoints.filename, breakpoints.filefullname, "
-            "breakpoints.linenumber, breakpoints.enabled, "
-            "breakpoints.condition from "
-            "breakpoints where breakpoints.sessionid = "
-            + UString::from_int (session.session_id ());
+      "breakpoints.linenumber, breakpoints.enabled, "
+      "breakpoints.condition, breakpoints.ignorecount,"
+      "breakpoints.iscountpoint from "
+      "breakpoints where breakpoints.sessionid = "
+      + UString::from_int (session.session_id ());
+
+    LOG_DD ("query: " << query);
     THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
     while (trans.get ().get_connection ().read_next_row ()) {
-        UString filename, filefullname, linenumber, enabled, condition;
+        UString filename, filefullname, linenumber,
+	  enabled, condition, ignorecount, is_countpoint;
         THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
                                                             (0, filename));
         THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
@@ -434,14 +499,46 @@ SessMgr::load_session (Session &a_session,
         THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
                                                             (4, condition));
         condition.chomp ();
-        session.breakpoints ().push_back (SessMgr::BreakPoint (filename,
+        THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                            (5, ignorecount));
+	THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                            (6, is_countpoint));
+        LOG_DD ("filename, filefullname, linenumber, enabled, "
+                "condition, ignorecount:\n"
+                << filename << "," << filefullname << ","
+                << linenumber << "," << enabled << ","
+                << condition << "," << ignorecount
+		<< is_countpoint);
+        session.breakpoints ().push_back (SessMgr::Breakpoint (filename,
                                                                filefullname,
                                                                linenumber,
                                                                enabled,
-                                                               condition));
+                                                               condition,
+                                                               ignorecount,
+							       is_countpoint));
     }
 
-    //load the search paths
+    // load the watchpoints
+    query = "select watchpoints.expression, watchpoints.iswrite, "
+            "watchpoints.isread from watchpoints where watchpoints.sessionid = "
+            + UString::from_int (session.session_id ());
+    LOG_DD ("query: " << query);
+    THROW_IF_FAIL (trans.get ().get_connection ().execute_statement (query));
+    while (trans.get ().get_connection ().read_next_row ()) {
+        UString expression;
+        gint64 is_write = false, is_read = false;
+        THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                            (0, expression));
+        THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                        (1, is_write));
+        THROW_IF_FAIL (trans.get ().get_connection ().get_column_content
+                                                            (2, is_read));
+        session.watchpoints ().push_back (SessMgr::WatchPoint (expression,
+                                                               is_write,
+                                                               is_read));
+    }
+
+    // load the search paths
     query = "select searchpaths.path from "
             "searchpaths where searchpaths.sessionid = "
             + UString::from_int (session.session_id ());
@@ -453,7 +550,7 @@ SessMgr::load_session (Session &a_session,
         session.search_paths ().push_back (path);
     }
 
-    //load the opened files
+    // load the opened files
     query = "select openedfiles.filename from openedfiles where "
             "openedfiles.sessionid = "
             + UString::from_int (session.session_id ());
