@@ -29,13 +29,13 @@ on_program_finished_signal ()
 }
 
 void
-on_breakpoints_set_signal (const std::map<int, IDebugger::BreakPoint> &a_breaks,
+on_breakpoints_set_signal (const std::map<int, IDebugger::Breakpoint> &a_breaks,
                            const UString &a_cookie)
 {
     if (a_cookie.empty ()) {}
 
     MESSAGE ("breakpoints set:");
-    std::map<int, IDebugger::BreakPoint>::const_iterator it;
+    std::map<int, IDebugger::Breakpoint>::const_iterator it;
     for (it = a_breaks.begin (); it != a_breaks.end () ; ++it) {
         MESSAGE ("<break><num>" << it->first <<"</num><line>"
                  << it->second.file_name () << ":" << it->second.line ()
